@@ -12,7 +12,7 @@ from fastel.collections import get_collection, get_site_config
 from fastel.config import SdkConfig
 from fastel.payment.common.models.callback import CallbackModel
 from fastel.payment.common.models.invoice import InvoiceIssueResp, InvoiceStatus
-from fastel.payment.common.models.order import Order
+from fastel.payment.common.models.order import Checkout, Order
 from fastel.payment.ecpay import (
     ECPAY_CALLBACK_SUBTYPE,
     generate_B2B_invoice_data,
@@ -40,8 +40,8 @@ from fastel.payment.neweb.models.callback import (
 )
 from fastel.payment.neweb.models.checkout import CheckoutModel
 from fastel.payment.neweb.models.invoice import IssueModel, IssueResp
-from fastel.payment.utils import PaymentStatus, auto_order_number
-from fastel.utils import TW, timestamp_postfix
+from fastel.payment.utils import PaymentStatus, auto_order_number, parse_order_number
+from fastel.utils import TW, default_timestamp, timestamp_postfix
 from fastel.utils.requests import parse_formdata
 
 
@@ -60,13 +60,16 @@ class NewebpayCheckout:
 
     def __call__(self, cart: Cart, **kwargs: Any) -> HTMLResponse:
         checkout_model = self.parse_payload(cart, **kwargs)
-        cart_dict = cart.get_cart_ins()
-        del cart_dict["_id"]
+        now = default_timestamp()
         checkout_ins = {
-            **cart_dict,
             "order_number": checkout_model.MerchantOrderNo,
+            "created": now,
+            "updated": now,
+            "expired": now + 7 * 24 * 3600 * 1000,
+            "attempt": 0,
+            "resolved": False,
         }
-        get_collection("checkout").insert_one(checkout_ins)
+        cart.to_checkout(**checkout_ins)
         return HTMLResponse(self.gateway.checkout_request(checkout_model))
 
     def parse_payload(self, cart: Cart, **kwargs: Any) -> CheckoutModel:
@@ -93,6 +96,49 @@ class NewebpayCheckout:
         )
 
 
+class NewebpayResume:
+    def __init__(self, callback_url: str, number_url: str, redirect_url: str) -> None:
+        self.gateway = Newebpay(
+            SdkConfig.neweb_merchant_id,
+            SdkConfig.neweb_hash_key,
+            SdkConfig.neweb_hash_iv,
+            stage=SdkConfig.stage,
+        )
+        self.notify_url = callback_url
+        self.customer_url = number_url
+        self.return_url = redirect_url
+
+    def __call__(self, checkout: Checkout, **kwargs: Any) -> HTMLResponse:
+        checkout_model = self.parse_payload(checkout, **kwargs)
+        return HTMLResponse(self.gateway.checkout_request(checkout_model))
+
+    def parse_payload(self, checkout: Checkout, **kwargs: Any) -> CheckoutModel:
+        type_map = {}
+        if checkout.payment_subtype == PaymentSubTypes.credit:
+            type_map["CREDIT"] = 1
+        elif checkout.payment_subtype == PaymentSubTypes.atm:
+            type_map["VACC"] = 1
+        elif checkout.payment_subtype == PaymentSubTypes.linepay:
+            type_map["LINEPAY"] = 1
+        elif checkout.payment_subtype == PaymentSubTypes.cod:
+            type_map["CVSCOM"] = 1
+
+        get_collection("checkout").update_one(
+            {"order_number": checkout.order_number}, {"$inc": {"attempt": 1}}
+        )
+        return CheckoutModel(
+            Amt=checkout.total,
+            ItemDesc=checkout.items[0].name,
+            MerchantOrderNo=f"{checkout.order_number}_{checkout.attempt}",
+            Email=checkout.buyer_email,
+            NotifyURL=self.notify_url,
+            CustomerURL=self.customer_url,
+            ReturnURL=self.return_url,
+            **type_map,
+            **kwargs,
+        )
+
+
 class NewebpayCallback:
     def __init__(self) -> None:
         self.gateway = Newebpay(
@@ -135,9 +181,10 @@ class NewebpayCallback:
         else:
             payment_subtype = PaymentSubTypes.unknown
 
+        parsered_order_number = parse_order_number(decrypted.Result.MerchantOrderNo)
         return CallbackModel(
             payment_status=payment_status,
-            order_id=decrypted.Result.MerchantOrderNo,
+            order_id=parsered_order_number,
             payment_subtype=payment_subtype,
             payment_transaction_detail=result.dict(),
             total=result.Amt,
@@ -221,7 +268,11 @@ class NewebpayGetNumber:
         )
 
     def respond(self, **kwargs: Any) -> Response:
-        return Response(content="1")
+        order_id = kwargs["order_id"]
+        return RedirectResponse(
+            url=f"{SdkConfig.web_host}/profile/orders?action=detail&is_created=true&order_number={order_id}",
+            status_code=302,
+        )
 
 
 class NewebInvoiceIssue:
@@ -372,13 +423,16 @@ class EcpayCheckout:
 
     def __call__(self, cart: Cart, **kwargs: Any) -> HTMLResponse:
         checkout_model = self.parse_payload(cart, **kwargs)
-        cart_dict = cart.get_cart_ins()
-        del cart_dict["_id"]
+        now = default_timestamp()
         checkout_ins = {
             "order_number": checkout_model.MerchantTradeNo,
-            **cart_dict,
+            "created": now,
+            "updated": now,
+            "expired": now + 7 * 24 * 3600 * 1000,
+            "attempt": 0,
+            "resolved": False,
         }
-        get_collection("checkout").insert_one(checkout_ins)
+        cart.to_checkout(**checkout_ins)
         return HTMLResponse(self.gateway.checkout_request(checkout_model))
 
     @staticmethod
@@ -430,6 +484,75 @@ class EcpayCheckout:
         )
 
 
+class EcpayResume:
+    def __init__(self, callback_url: str, number_url: str, redirect_url: str) -> None:
+        self.gateway = Ecpay(
+            SdkConfig.ecpay_merchant_id,
+            SdkConfig.ecpay_hash_key,
+            SdkConfig.ecpay_hash_iv,
+            stage=SdkConfig.stage,
+        )
+        self.return_url = callback_url
+        self.payment_info_url = number_url
+        self.order_result_url = redirect_url
+
+    def __call__(self, checkout: Checkout, **kwargs: Any) -> HTMLResponse:
+        checkout_model = self.parse_payload(checkout, **kwargs)
+        return HTMLResponse(self.gateway.checkout_request(checkout_model))
+
+    @staticmethod
+    def _replace_limit_name(name: str, limit: int) -> str:
+        if len(name) > limit:
+            return name[: limit - 3] + "..."
+        return name
+
+    def parse_payload(self, checkout: Checkout, **kwargs: Any) -> EcCheckoutModel:
+        now = datetime.now(TW)
+        type_map = {}
+        if checkout.payment_subtype == PaymentSubTypes.credit:
+            type_map["ChoosePayment"] = "Credit"
+            type_map["OrderResultURL"] = self.order_result_url
+        elif checkout.payment_subtype == PaymentSubTypes.webatm:
+            type_map["ChoosePayment"] = "WebATM"
+            type_map["OrderResultURL"] = self.order_result_url
+        elif checkout.payment_subtype == PaymentSubTypes.atm:
+            type_map["ChoosePayment"] = "ATM"
+            type_map["PaymentInfoURL"] = self.payment_info_url
+            type_map["ClientRedirectURL"] = self.order_result_url
+        elif checkout.payment_subtype == PaymentSubTypes.cvs:
+            type_map["ChoosePayment"] = "CVS"
+            type_map["PaymentInfoURL"] = self.payment_info_url
+            type_map["ClientRedirectURL"] = self.order_result_url
+        elif checkout.payment_subtype == PaymentSubTypes.barcode:
+            type_map["ChoosePayment"] = "BARCODE"
+            type_map["PaymentInfoURL"] = self.payment_info_url
+            type_map["ClientRedirectURL"] = self.order_result_url
+        elif checkout.payment_subtype == PaymentSubTypes.default:
+            type_map["ChoosePayment"] = "ALL"
+            type_map["PaymentInfoURL"] = self.payment_info_url
+            type_map["ClientRedirectURL"] = self.order_result_url
+            type_map["OrderResultURL"] = self.order_result_url
+        else:
+            raise ValueError("invalid Payment Subtypes")
+
+        name = "#".join([item.name for item in checkout.items])
+        item_name = self._replace_limit_name(name=name, limit=400)
+
+        get_collection("checkout").update_one(
+            {"order_number": checkout.order_number}, {"$inc": {"attempt": 1}}
+        )
+
+        return EcCheckoutModel(
+            TotalAmount=checkout.total,
+            ItemName=item_name,
+            MerchantTradeNo=f"{checkout.order_number}_{checkout.attempt}",
+            MerchantTradeDate=now.strftime("%Y/%m/%d %H:%M:%S"),
+            ReturnURL=self.return_url,
+            **type_map,
+            **kwargs,
+        )
+
+
 class EcpayCallback:
     def __init__(self) -> None:
         self.gateway = Ecpay(
@@ -470,9 +593,11 @@ class EcpayCallback:
 
         payment_type = result.PaymentType.split("_")[0]
 
+        parsered_order_number = parse_order_number(result.MerchantTradeNo)
+
         return CallbackModel(
             payment_status=payment_status,
-            order_id=result.MerchantTradeNo,
+            order_id=parsered_order_number,
             total=result.TradeAmt,
             payment_subtype=ECPAY_CALLBACK_SUBTYPE[payment_type],
             payment_transaction_id=result.TradeNo,
@@ -635,13 +760,16 @@ class LinePayCheckout:
         checkout_model = self.parse_payload(cart, **kwargs)
         checkout_resp = self.gateway.checkout_request(checkout_model)
         response = self.parse_gateway_resp(checkout_resp)
-        cart_dict = cart.get_cart_ins()
-        del cart_dict["_id"]
+        now = default_timestamp()
         checkout_ins = {
-            **cart_dict,
             "order_number": checkout_model.orderId,
+            "created": now,
+            "updated": now,
+            "expired": now + 7 * 24 * 3600 * 1000,
+            "attempt": 0,
+            "resolved": False,
         }
-        get_collection("checkout").insert_one(checkout_ins)
+        cart.to_checkout(**checkout_ins)
         return response
 
     def parse_gateway_resp(self, gateway_resp: LinePayRequestResp) -> Response:
@@ -650,7 +778,7 @@ class LinePayCheckout:
         else:
             redirect_url = self.error_url
 
-        return Response(content={"url": redirect_url})
+        return RedirectResponse(url=redirect_url, status_code=302)
 
     def parse_payload(self, cart: Cart, **kwargs: Any) -> LinePayCheckoutModel:
         if cart.payment_subtype != PaymentSubTypes.linepay:
@@ -707,7 +835,7 @@ class LinePayCallback:
         callback_model: LinePayCallbackModel,
     ) -> LinePayConfirmModel:
         checkout_dict = get_collection("checkout").find_one(
-            {"order_number", callback_model.orderId}
+            {"order_number": callback_model.orderId}
         )
         return LinePayConfirmModel(
             amount=checkout_dict["total"], transactionId=callback_model.transactionId
@@ -745,6 +873,26 @@ class LinePayCallback:
         )
 
 
+def get_resume_method(
+    prefix_path: str = "payment",
+) -> Union[NewebpayResume, EcpayResume]:
+    default_payment_provider = get_site_config("default_payment_provider", "ecpay")
+    if default_payment_provider == "neweb":
+        return NewebpayResume(
+            callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
+            number_url=f"{SdkConfig.api_host}/{prefix_path}/number",
+            redirect_url=f"{SdkConfig.api_host}/{prefix_path}/return",
+        )
+    elif default_payment_provider == "ecpay":
+        return EcpayResume(
+            callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
+            number_url=f"{SdkConfig.api_host}/{prefix_path}/number",
+            redirect_url=f"{SdkConfig.api_host}/{prefix_path}/return",
+        )
+
+    raise ValueError("unrecognize payment provider")
+
+
 def get_checkout_method(
     prefix_path: str = "payment",
 ) -> Union[NewebpayCheckout, EcpayCheckout, LinePayCheckout]:
@@ -764,7 +912,7 @@ def get_checkout_method(
     elif default_payment_provider == "linepay":
         return LinePayCheckout(
             callback_url=f"{SdkConfig.api_host}/{prefix_path}/callback",
-            error_url=f"{SdkConfig.web_host}/misc?title=結帳發生錯誤&url={SdkConfig.web_host}&btn_text=返回",
+            error_url=f"{SdkConfig.web_host}/misc/error?title=結帳發生錯誤&url={SdkConfig.web_host}&btn_text=返回首頁",
         )
 
     raise ValueError("unrecognize payment provider")
@@ -839,7 +987,7 @@ async def process_getnum(request: Request) -> Response:
         input=getnum_result.json(),
     )
 
-    return getnum_method.respond()
+    return getnum_method.respond(order_id=getnum_result.order_id)
 
 
 def process_invoice_issue(order: Dict[str, Any]) -> InvoiceIssueResp:
