@@ -6,7 +6,7 @@ from pydantic import BaseModel, EmailStr
 class CheckoutModel(BaseModel):
     MerchantOrderNo: str
     Amt: int
-    ItemDesc: str = "Online Payment To Ecpay"
+    ItemDesc: str = "Online Payment To Neweb pay"
     Email: EmailStr
     CREDIT: Optional[Literal[0, 1]]
     ANDROIDPAY: Optional[Literal[0, 1]]
@@ -27,3 +27,8 @@ class CheckoutModel(BaseModel):
     NotifyURL: Optional[str]
     ReturnURL: Optional[str]
     CustomerURL: Optional[str]
+
+    MerchantID: Optional[str]
+    Version: Optional[str] = "1.5"
+    RespondType: Optional[str] = "JSON"
+    TimeStamp: Optional[str]
