@@ -1,7 +1,8 @@
 from functools import wraps
-from typing import Any, Callable, TypeVar, cast
+from typing import Any, Callable, Dict, TypeVar, cast
 
 import requests as _requests
+from starlette.datastructures import FormData
 
 FuncT = TypeVar("FuncT", bound=Callable[..., Any])
 WrapT = TypeVar("WrapT", bound=Callable[..., Any])
@@ -40,3 +41,11 @@ def put(*args: Any, **kwargs: Any) -> Any:
 @log_all("delete")  # type: ignore
 def delete(*args: Any, **kwargs: Any) -> Any:
     return _requests.delete(*args, **kwargs)
+
+
+def parse_formdata(formdata: FormData) -> Dict[str, Any]:
+    parsed = {}
+    for key in formdata.keys():
+        parsed[key] = formdata.get(key)
+
+    return parsed
